@@ -22,12 +22,14 @@ module Puppet # :nodoc:
 
         def to_s
             str = nil
-            if defined? @file and defined? @line and @file and @line
-                str = "%s in file %s at line %s" %
+            if self.file and self.line
+                str = "%s at %s:%s" %
                     [@message.to_s, @file, @line]
-            elsif defined? @line and @line
+            elsif self.line
                 str = "%s at line %s" %
                     [@message.to_s, @line]
+            elsif self.file
+                str = "%s in %s" % [@message.to_s, self.file]
             else
                 str = @message.to_s
             end
@@ -39,14 +41,5 @@ module Puppet # :nodoc:
     # An error class for when I don't know what happened.  Automatically
     # prints a stack trace when in debug mode.
     class DevError < Puppet::Error
-        # XXX This is probably the wrong way to do this, but...
-        def set_backtrace(trace)
-            if Puppet[:trace]
-                puts trace
-            end
-            super
-        end
     end
 end
-
-# $Id: error.rb 1739 2006-10-06 03:13:15Z luke $
