@@ -1,13 +1,14 @@
 # The client for interacting with the puppetmaster config server.
 require 'sync'
 require 'timeout'
+require 'puppet/network/http_pool'
 
 class Puppet::Network::Client::Master < Puppet::Network::Client
     unless defined? @@sync
         @@sync = Sync.new
     end
 
-    attr_accessor :objects
+    attr_accessor :catalog
     attr_reader :compile_time
 
     class << self
@@ -25,69 +26,37 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
         
         down = Puppet[:downcasefacts]
 
-        facts = {}
-        Facter.each { |name,fact|
+        facts = Facter.to_hash.inject({}) do |newhash, array|
+            name, fact = array
             if down
-                facts[name] = fact.to_s.downcase
+                newhash[name] = fact.to_s.downcase
             else
-                facts[name] = fact.to_s
+                newhash[name] = fact.to_s
             end
-        }
+            newhash
+        end
 
         # Add our client version to the list of facts, so people can use it
         # in their manifests
         facts["clientversion"] = Puppet.version.to_s
 
+        # And add our environment as a fact.
+        unless facts.include?("environment")
+            facts["environment"] = Puppet[:environment]
+        end
+ 
         facts
     end
 
-    # This method actually applies the configuration.
-    def apply(tags = nil, ignoreschedules = false)
-        unless defined? @objects
-            raise Puppet::Error, "Cannot apply; objects not defined"
-        end
-
-        transaction = @objects.evaluate
-
-        if tags
-            transaction.tags = tags
-        end
-
-        if ignoreschedules
-            transaction.ignoreschedules = true
-        end
-
-        transaction.addtimes :config_retrieval => @configtime
-
-        begin
-            transaction.evaluate
-        rescue Puppet::Error => detail
-            Puppet.err "Could not apply complete configuration: %s" %
-                detail
-        rescue => detail
-            Puppet.err "Got an uncaught exception of type %s: %s" %
-                [detail.class, detail]
-            if Puppet[:trace]
-                puts detail.backtrace
-            end
-        ensure
-            Puppet::Util::Storage.store
-        end
-        
-        if Puppet[:report]
-            report(transaction)
-        end
-
-        return transaction
-    ensure
-        if defined? transaction and transaction
-            transaction.cleanup
-        end
+    # Return the list of dynamic facts as an array of symbols
+    def self.dynamic_facts
+        # LAK:NOTE See http://snurl.com/21zf8  [groups_google_com] 
+        x = Puppet.settings[:dynamicfacts].split(/\s*,\s*/).collect { |fact| fact.downcase }
     end
 
     # Cache the config
     def cache(text)
-        Puppet.info "Caching configuration at %s" % self.cachefile
+        Puppet.info "Caching catalog at %s" % self.cachefile
         confdir = ::File.dirname(Puppet[:localconfig])
         ::File.open(self.cachefile + ".tmp", "w", 0660) { |f|
             f.print text
@@ -103,10 +72,9 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
     end
 
     def clear
-        @objects.remove(true)
+        @catalog.clear(true) if @catalog
         Puppet::Type.allclear
-        mkdefault_objects
-        @objects = nil
+        @catalog = nil
     end
 
     # Initialize and load storage
@@ -129,7 +97,7 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
         end
     end
 
-    # Check whether our configuration is up to date
+    # Check whether our catalog is up to date
     def fresh?(facts)
         if Puppet[:ignorecache]
             Puppet.notice "Ignoring cache"
@@ -144,12 +112,12 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
             return false
         end
 
-        # We're willing to give a 2 second drift
         newcompile = @driver.freshness
+        # We're willing to give a 2 second drift
         if newcompile - @compile_time.to_i < 1
             return true
         else
-            Puppet.debug "Server compile time is %s vs %s" % [newcompile, @compile_time]
+            Puppet.debug "Server compile time is %s vs %s" % [newcompile, @compile_time.to_i]
             return false
         end
     end
@@ -160,7 +128,7 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
         lockfile.unlock(:anonymous => true)
     end
 
-    # Stop the daemon from making any configuration runs.
+    # Stop the daemon from making any catalog runs.
     def disable
         lockfile.lock(:anonymous => true)
     end
@@ -170,65 +138,62 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
     def getconfig
         dostorage()
 
-        facts = self.class.facts
-
-        if self.objects or FileTest.exists?(self.cachefile)
-            if self.fresh?(facts)
-                Puppet.info "Config is up to date"
-                if self.objects
-                    return
-                end
-                if oldtext = self.retrievecache
-                    begin
-                        @objects = YAML.load(oldtext).to_type
-                    rescue => detail
-                        Puppet.warning "Could not load cached configuration: %s" % detail
-                    end
-                    return
-                end
-            end
+        facts = nil
+        Puppet::Util.benchmark(:debug, "Retrieved facts") do
+            facts = self.class.facts
         end
-        Puppet.debug("getting config")
+
+        raise Puppet::Network::ClientError.new("Could not retrieve any facts") unless facts.length > 0
 
         # Retrieve the plugins.
-        if Puppet[:pluginsync]
-            getplugins()
+        getplugins() if Puppet[:pluginsync]
+
+        if (self.catalog or FileTest.exist?(self.cachefile)) and self.fresh?(facts)
+            Puppet.info "Configuration is up to date"
+            return if use_cached_config
         end
 
-        unless facts.length > 0
-            raise Puppet::Network::ClientError.new(
-                "Could not retrieve any facts"
-            )
-        end
+        Puppet.debug("Retrieving catalog")
 
-        unless objects = get_actual_config(facts)
-            @objects = nil
+        # If we can't retrieve the catalog, just return, which will either
+        # fail, or use the in-memory catalog.
+        unless yaml_objects = get_actual_config(facts)
+            use_cached_config(true)
             return
         end
 
-        unless objects.is_a?(Puppet::TransBucket)
-            raise NetworkClientError,
-                "Invalid returned objects of type %s" % objects.class
+        begin
+            objects = YAML.load(yaml_objects)
+        rescue => detail
+            msg = "Configuration could not be translated from yaml"
+            msg += "; using cached catalog" if use_cached_config(true)
+            Puppet.warning msg
+            return
         end
 
         self.setclasses(objects.classes)
 
         # Clear all existing objects, so we can recreate our stack.
-        if self.objects
+        clear() if self.catalog
+
+        # Now convert the objects to a puppet catalog graph.
+        begin
+            @catalog = objects.to_catalog
+        rescue => detail
             clear()
+            puts detail.backtrace if Puppet[:trace]
+            msg = "Configuration could not be instantiated: %s" % detail
+            msg += "; using cached catalog" if use_cached_config(true)
+            Puppet.warning msg
+            return
         end
 
-        # Now convert the objects to real Puppet objects
-        @objects = objects.to_type
-
-        if @objects.nil?
-            raise Puppet::Error, "Configuration could not be processed"
+        if ! @catalog.from_cache
+            self.cache(yaml_objects)
         end
 
-        # and perform any necessary final actions before we evaluate.
-        @objects.finalize
-
-        return @objects
+        # Keep the state database up to date.
+        @catalog.host_config = true
     end
     
     # A simple proxy method, so it's easy to test.
@@ -238,25 +203,11 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
     
     # Just so we can specify that we are "the" instance.
     def initialize(*args)
-        Puppet.config.use(:puppet, :sslcertificates, :puppetd)
+        Puppet.settings.use(:main, :ssl, :puppetd)
         super
 
-        # This might be nil
-        @configtime = 0
-
         self.class.instance = self
         @running = false
-
-        mkdefault_objects
-    end
-
-    # Make the default objects necessary for function.
-    def mkdefault_objects
-        # First create the default scheduling objects
-        Puppet::Type.type(:schedule).mkdefaultschedules
-        
-        # And filebuckets
-        Puppet::Type.type(:filebucket).mkdefaultbucket
     end
 
     # Mark that we should restart.  The Puppet module checks whether we're running,
@@ -285,27 +236,38 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
         end
     end
 
-    # The code that actually runs the configuration.  
-    def run(tags = nil, ignoreschedules = false)
+    # The code that actually runs the catalog.  
+    # This just passes any options on to the catalog,
+    # which accepts :tags and :ignoreschedules.
+    def run(options = {})
         got_lock = false
+        splay
         Puppet::Util.sync(:puppetrun).synchronize(Sync::EX) do
             if !lockfile.lock
-                Puppet.notice "Lock file %s exists; skipping configuration run" %
+                Puppet.notice "Lock file %s exists; skipping catalog run" %
                     lockfile.lockfile
             else
                 got_lock = true
-                @configtime = thinmark do
-                    self.getconfig
+                begin
+                    duration = thinmark do
+                        self.getconfig
+                    end
+                rescue => detail
+                    puts detail.backtrace if Puppet[:trace]
+                    Puppet.err "Could not retrieve catalog: %s" % detail
                 end
 
-                if defined? @objects and @objects
-                    unless @local
-                        Puppet.notice "Starting configuration run"
-                    end
-                    benchmark(:notice, "Finished configuration run") do
-                        self.apply(tags, ignoreschedules)
+                if self.catalog
+                    @catalog.retrieval_duration = duration
+                    Puppet.notice "Starting catalog run" unless @local
+                    benchmark(:notice, "Finished catalog run") do
+                        @catalog.apply(options)
                     end
                 end
+
+                # Now close all of our existing http connections, since there's no
+                # reason to leave them lying open.
+                Puppet::Network::HttpPool.clear_http_instances
             end
             
             lockfile.unlock
@@ -319,6 +281,7 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
     ensure
         # Just make sure we remove the lock file if we set it.
         lockfile.unlock if got_lock and lockfile.locked?
+        clear()
     end
 
     def running?
@@ -349,9 +312,6 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
     # Download files from the remote server, returning a list of all
     # changed files.
     def self.download(args)
-        objects = Puppet::Type.type(:component).create(
-            :name => "#{args[:name]}_collector"
-        )
         hash = {
             :path => args[:dest],
             :recurse => true,
@@ -360,24 +320,28 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
             :owner => Process.uid,
             :group => Process.gid,
             :purge => true,
-            :backup => false
+            :force => true,
+            :backup => false,
+            :noop => false
         }
 
         if args[:ignore]
             hash[:ignore] = args[:ignore].split(/\s+/)
         end
-        objects.push Puppet::Type.type(:file).create(hash)
+        downconfig = Puppet::Node::Catalog.new("downloading")
+        downconfig.add_resource Puppet::Type.type(:file).create(hash)
         
         Puppet.info "Retrieving #{args[:name]}s"
 
-        noop = Puppet[:noop]
-        Puppet[:noop] = false
-
+        files = []
         begin
-            trans = objects.evaluate
-            trans.ignoretags = true
             Timeout::timeout(self.timeout) do
-                trans.evaluate
+                downconfig.apply do |trans|
+                    trans.changed?.find_all do |resource|
+                        yield resource if block_given?
+                        files << resource[:path]
+                    end
+                end
             end
         rescue Puppet::Error, Timeout::Error => detail
             if Puppet[:debug]
@@ -386,45 +350,28 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
             Puppet.err "Could not retrieve #{args[:name]}s: %s" % detail
         end
 
-        # Now source all of the changed objects, but only source those
-        # that are top-level.
-        files = []
-        trans.changed?.find_all do |object|
-            yield object if block_given?
-            files << object[:path]
-        end
-        trans.cleanup
-
         # Now clean up after ourselves
-        objects.remove
-        files
-    ensure
-        # I can't imagine why this is necessary, but apparently at last one person has had problems with noop
-        # being nil here.
-        if noop.nil?
-            Puppet[:noop] = false
-        else
-            Puppet[:noop] = noop
-        end
+        downconfig.clear
+
+        return files
     end
 
     # Retrieve facts from the central server.
     def self.getfacts
-        # Clear all existing definitions.
-        Facter.clear
-
         # Download the new facts
         path = Puppet[:factpath].split(":")
         files = []
         download(:dest => Puppet[:factdest], :source => Puppet[:factsource],
-            :ignore => Puppet[:factsignore], :name => "fact") do |object|
+            :ignore => Puppet[:factsignore], :name => "fact") do |resource|
 
-            next unless path.include?(::File.dirname(object[:path]))
-
-            files << object[:path]
+            next unless path.include?(::File.dirname(resource[:path]))
 
+            files << resource[:path]
         end
     ensure
+        # Clear all existing definitions.
+        Facter.clear
+
         # Reload everything.
         if Facter.respond_to? :loadfacts
             Facter.loadfacts
@@ -443,19 +390,21 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
     # Retrieve the plugins from the central server.  We only have to load the
     # changed plugins, because Puppet::Type loads plugins on demand.
     def self.getplugins
-        path = Puppet[:pluginpath].split(":")
         download(:dest => Puppet[:plugindest], :source => Puppet[:pluginsource],
-            :ignore => Puppet[:pluginsignore], :name => "plugin") do |object|
+            :ignore => Puppet[:pluginsignore], :name => "plugin") do |resource|
 
-            next unless path.include?(::File.dirname(object[:path]))
+            next if FileTest.directory?(resource[:path])
+            path = resource[:path].sub(Puppet[:plugindest], '').sub(/^\/+/, '')
+            unless Puppet::Util::Autoload.loaded?(path)
+                next
+            end
 
             begin
-                Puppet.info "Reloading plugin %s" %
-                    ::File.basename(::File.basename(object[:path])).sub(".rb",'')
-                load object[:path]
+                Puppet.info "Reloading downloaded file %s" % path
+                load resource[:path]
             rescue => detail
-                Puppet.warning "Could not reload plugin %s: %s" %
-                    [object[:path], detail]
+                Puppet.warning "Could not reload downloaded file %s: %s" %
+                    [resource[:path], detail]
             end
         end
     end
@@ -477,17 +426,18 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
     end
 
     def self.loadfacts
-        Puppet[:factpath].split(":").each do |dir|
+        # LAK:NOTE See http://snurl.com/21zf8  [groups_google_com] 
+        x = Puppet[:factpath].split(":").each do |dir|
             loaddir(dir, "fact")
         end
     end
     
     def self.timeout
-        @timeout = Puppet[:configtimeout]
-        case @timeout
+        timeout = Puppet[:configtimeout]
+        case timeout
         when String:
-            if @timeout =~ /^\d+$/
-                @timeout = Integer(@timeout)
+            if timeout =~ /^\d+$/
+                timeout = Integer(timeout)
             else
                 raise ArgumentError, "Configuration timeout must be an integer"
             end
@@ -495,39 +445,22 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
         else
             raise ArgumentError, "Configuration timeout must be an integer"
         end
-    end
-    
-    # Send off the transaction report.
-    def report(transaction)
-        begin
-            report = transaction.generate_report()
-            if Puppet[:rrdgraph] == true
-                report.graph()
-            end
-            reportclient().report(report)
-        rescue => detail
-            Puppet.err "Reporting failed: %s" % detail
-        end
-    end
 
-    def reportclient
-        unless defined? @reportclient
-            @reportclient = Puppet::Network::Client.report.new(
-                :Server => Puppet[:reportserver]
-            )
-        end
-
-        @reportclient
+        return timeout
     end
 
     loadfacts()
-    
-    private
 
     # Have the facts changed since we last compiled?
     def facts_changed?(facts)
-        oldfacts = Puppet::Util::Storage.cache(:configuration)[:facts]
-        newfacts = facts
+        oldfacts = (Puppet::Util::Storage.cache(:configuration)[:facts] || {}).dup
+        newfacts = facts.dup
+        self.class.dynamic_facts.each do |fact|
+            [oldfacts, newfacts].each do |facthash|
+                facthash.delete(fact) if facthash.include?(fact)
+            end
+        end
+
         if oldfacts == newfacts
             return false
         else
@@ -544,92 +477,48 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
         end
     end
     
-    # Actually retrieve the configuration, either from the server or from a
+    # Actually retrieve the catalog, either from the server or from a
     # local master.
     def get_actual_config(facts)
-        if @local
-            return get_local_config(facts)
-        else
-            begin
-                Timeout::timeout(self.class.timeout) do
-                    return get_remote_config(facts)
-                end
-            rescue Timeout::Error
-                Puppet.err "Configuration retrieval timed out"
-                return nil
+        begin
+            Timeout::timeout(self.class.timeout) do
+                return get_remote_config(facts)
             end
+        rescue Timeout::Error
+            Puppet.err "Configuration retrieval timed out"
+            return nil
         end
     end
     
-    # Retrieve a configuration from a local master.
-    def get_local_config(facts)
-        # If we're local, we don't have to do any of the conversion
-        # stuff.
-        objects = @driver.getconfig(facts, "yaml")
-        @compile_time = Time.now
-
-        if objects == ""
-            raise Puppet::Error, "Could not retrieve configuration"
-        end
-        
-        return objects
-    end
-    
     # Retrieve a config from a remote master.
     def get_remote_config(facts)
         textobjects = ""
 
         textfacts = CGI.escape(YAML.dump(facts))
 
-        benchmark(:debug, "Retrieved configuration") do
+        benchmark(:debug, "Retrieved catalog") do
             # error handling for this is done in the network client
             begin
                 textobjects = @driver.getconfig(textfacts, "yaml")
                 begin
                     textobjects = CGI.unescape(textobjects)
                 rescue => detail
-                    raise Puppet::Error, "Could not CGI.unescape configuration"
+                    raise Puppet::Error, "Could not CGI.unescape catalog"
                 end
 
             rescue => detail
-                Puppet.err "Could not retrieve configuration: %s" % detail
-
-                unless Puppet[:usecacheonfailure]
-                    @objects = nil
-                    Puppet.warning "Not using cache on failed configuration"
-                    return
-                end
+                Puppet.err "Could not retrieve catalog: %s" % detail
+                return nil
             end
         end
 
-        fromcache = false
-        if textobjects == ""
-            unless textobjects = self.retrievecache
-                raise Puppet::Error.new(
-                    "Cannot connect to server and there is no cached configuration"
-                )
-            end
-            Puppet.warning "Could not get config; using cached copy"
-            fromcache = true
-        else
-            @compile_time = Time.now
-            Puppet::Util::Storage.cache(:configuration)[:facts] = facts
-            Puppet::Util::Storage.cache(:configuration)[:compile_time] = @compile_time
-        end
+        return nil if textobjects == ""
 
-        begin
-            objects = YAML.load(textobjects)
-        rescue => detail
-            raise Puppet::Error,
-                "Could not understand configuration: %s" %
-                detail.to_s
-        end
+        @compile_time = Time.now
+        Puppet::Util::Storage.cache(:configuration)[:facts] = facts
+        Puppet::Util::Storage.cache(:configuration)[:compile_time] = @compile_time
 
-        if @cache and ! fromcache
-            self.cache(textobjects)
-        end
-        
-        return objects
+        return textobjects
     end
 
     def lockfile
@@ -639,6 +528,48 @@ class Puppet::Network::Client::Master < Puppet::Network::Client
 
         @lockfile
     end
-end
 
-# $Id: master.rb 2435 2007-04-30 16:19:37Z luke $
+    # Sleep when splay is enabled; else just return.
+    def splay
+        return unless Puppet[:splay]
+
+        limit = Integer(Puppet[:splaylimit])
+
+        # Pick a splay time and then cache it.
+        unless time = Puppet::Util::Storage.cache(:configuration)[:splay_time]
+            time = rand(limit)
+            Puppet::Util::Storage.cache(:configuration)[:splay_time] = time
+        end
+
+        Puppet.info "Sleeping for %s seconds (splay is enabled)" % time
+        sleep(time)
+    end
+
+    private
+
+    # Use our cached config, optionally specifying whether this is
+    # necessary because of a failure.
+    def use_cached_config(because_of_failure = false)
+        return true if self.catalog
+
+        if because_of_failure and ! Puppet[:usecacheonfailure]
+            @catalog = nil
+            Puppet.warning "Not using cache on failed catalog"
+            return false
+        end
+
+        return false unless oldtext = self.retrievecache
+
+        begin
+            @catalog = YAML.load(oldtext).to_catalog
+            @catalog.from_cache = true
+            @catalog.host_config = true
+        rescue => detail
+            puts detail.backtrace if Puppet[:trace]
+            Puppet.warning "Could not load cached catalog: %s" % detail
+            clear
+            return false
+        end
+        return true
+    end
+end
