@@ -1,4 +1,6 @@
-Puppet::Type.type(:package).provide :darwinport do
+require 'puppet/provider/package'
+
+Puppet::Type.type(:package).provide :darwinport, :parent => Puppet::Provider::Package do
     desc "Package management using DarwinPorts on OS X."
 
     commands :port => "/opt/local/bin/port"
@@ -31,31 +33,30 @@ Puppet::Type.type(:package).provide :darwinport do
         }
     end
 
-    def self.list
+    def self.instances
         packages = []
 
         eachpkgashash do |hash|
-            pkg = Puppet.type(:package).installedpkg(hash)
-            packages << pkg
+            packages << new(hash)
         end
 
         return packages
     end
 
     def install
-        should = @model.should(:ensure)
+        should = @resource.should(:ensure)
 
         # Seems like you can always say 'upgrade'
-        output = port "upgrade", @model[:name]
+        output = port "upgrade", @resource[:name]
         if output =~ /^Error: No port/
-            raise Puppet::ExecutionFailure, "Could not find package %s" % @model[:name]
+            raise Puppet::ExecutionFailure, "Could not find package %s" % @resource[:name]
         end
     end
 
     def query
         version = nil
         self.class.eachpkgashash do |hash|
-            if hash[:name] == @model[:name]
+            if hash[:name] == @resource[:name]
                 return hash
             end
         end
@@ -64,7 +65,7 @@ Puppet::Type.type(:package).provide :darwinport do
     end
 
     def latest
-        info = port :search, "^#{@model[:name]}$"
+        info = port :search, "^#{@resource[:name]}$"
 
         if $? != 0 or info =~ /^Error/
             return nil
@@ -77,7 +78,7 @@ Puppet::Type.type(:package).provide :darwinport do
     end
 
     def uninstall
-        port :uninstall, @model[:name]
+        port :uninstall, @resource[:name]
     end
 
     def update
@@ -85,4 +86,3 @@ Puppet::Type.type(:package).provide :darwinport do
     end
 end
 
-# $Id: darwinport.rb 2052 2007-01-05 17:09:04Z luke $
