@@ -1,15 +1,18 @@
 Puppet.type(:package).provide :rug, :parent => :rpm do
     desc "Support for suse ``rug`` package manager."
 
+    has_feature :versionable
+
     commands :rug => "/usr/bin/rug"
+    commands :rpm => "rpm"
     defaultfor :operatingsystem => :suse 
     confine    :operatingsystem => :suse
 
     # Install a package using 'rug'.
     def install
-        should = @model.should(:ensure)
+        should = @resource.should(:ensure)
         self.debug "Ensuring => #{should}"
-        wanted = @model[:name]
+        wanted = @resource[:name]
 
         # XXX: We don't actually deal with epochs here.
         case should
@@ -33,12 +36,12 @@ Puppet.type(:package).provide :rug, :parent => :rpm do
         #rug can only get a list of *all* available packages?
         output = rug "list-updates"
 
-        if output =~ /#{@model[:name]}\s*\|\s*([0-9\.\-]+)/
+        if output =~ /#{@resource[:name]}\s*\|\s*([0-9\.\-]+)/
             return $1
         else
             # rug didn't find updates, pretend the current
             # version is the latest
-            return @model.is(:ensure)
+            return @property_hash[:ensure]
         end
     end
 
@@ -46,8 +49,4 @@ Puppet.type(:package).provide :rug, :parent => :rpm do
         # rug install can be used for update, too
         self.install
     end
-
-    def versionable?
-        true
-    end
 end
