@@ -1,86 +1,11 @@
+require 'sync'
+
 class Puppet::SSLCertificates::CA
     include Puppet::Util::Warnings
 
     Certificate = Puppet::SSLCertificates::Certificate
     attr_accessor :keyfile, :file, :config, :dir, :cert, :crl
 
-    Puppet.setdefaults(:ca,
-        :cadir => {  :default => "$ssldir/ca",
-            :owner => "$user",
-            :group => "$group",
-            :mode => 0770,
-            :desc => "The root directory for the certificate authority."
-        },
-        :cacert => { :default => "$cadir/ca_crt.pem",
-            :owner => "$user",
-            :group => "$group",
-            :mode => 0660,
-            :desc => "The CA certificate."
-        },
-        :cakey => { :default => "$cadir/ca_key.pem",
-            :owner => "$user",
-            :group => "$group",
-            :mode => 0660,
-            :desc => "The CA private key."
-        },
-        :capub => { :default => "$cadir/ca_pub.pem",
-            :owner => "$user",
-            :group => "$group",
-            :desc => "The CA public key."
-        },
-        :cacrl => { :default => "$cadir/ca_crl.pem",
-            :owner => "$user",
-            :group => "$group",
-            :mode => 0664,
-            :desc => "The certificate revocation list (CRL) for the CA. Set this to 'none' if you do not want to use a CRL."
-        },
-        :caprivatedir => { :default => "$cadir/private",
-            :owner => "$user",
-            :group => "$group",
-            :mode => 0770,
-            :desc => "Where the CA stores private certificate information."
-        },
-        :csrdir => { :default => "$cadir/requests",
-            :owner => "$user",
-            :group => "$group",
-            :desc => "Where the CA stores certificate requests"
-        },
-        :signeddir => { :default => "$cadir/signed",
-            :owner => "$user",
-            :group => "$group",
-            :mode => 0770,
-            :desc => "Where the CA stores signed certificates."
-        },
-        :capass => { :default => "$caprivatedir/ca.pass",
-            :owner => "$user",
-            :group => "$group",
-            :mode => 0660,
-            :desc => "Where the CA stores the password for the private key"
-        },
-        :serial => { :default => "$cadir/serial",
-            :owner => "$user",
-            :group => "$group",
-            :desc => "Where the serial number for certificates is stored."
-        },
-        :autosign => { :default => "$confdir/autosign.conf",
-            :mode => 0644,
-            :desc => "Whether to enable autosign.  Valid values are true (which
-                autosigns any key request, and is a very bad idea), false (which
-                never autosigns any key request), and the path to a file, which
-                uses that configuration file to determine which keys to sign."},
-        :ca_days => ["", "How long a certificate should be valid. 
-                 This parameter is deprecated, use ca_ttl instead"],
-        :ca_ttl => ["5y", "The default TTL for new certificates; valid values 
-                must be an integer, optionally followed by one of the units 
-                'y' (years of 365 days), 'd' (days), 'h' (hours), or 
-                's' (seconds). The unit defaults to seconds. If this parameter
-                is set, ca_days is ignored. Examples are '3600' (one hour) 
-                and '1825d', which is the same as '5y' (5 years) "],
-        :ca_md => ["md5", "The type of hash used in certificates."],
-        :req_bits => [2048, "The bit length of the certificates."],
-        :keylength => [1024, "The bit length of keys."]
-    )
-
     def certfile
         @config[:cacert]
     end
@@ -128,7 +53,7 @@ class Puppet::SSLCertificates::CA
     end
 
     def initialize(hash = {})
-        Puppet.config.use(:puppet, :certificates, :ca)
+        Puppet.settings.use(:main, :ca, :ssl)
         self.setconfig(hash)
 
         if Puppet[:capass]
@@ -148,7 +73,7 @@ class Puppet::SSLCertificates::CA
         self.getcert
         init_crl
         unless FileTest.exists?(@config[:serial])
-            Puppet.config.write(:serial) do |f|
+            Puppet.settings.write(:serial) do |f|
                 f << "%04X" % 1
             end
         end
@@ -160,7 +85,7 @@ class Puppet::SSLCertificates::CA
         20.times { pass += (rand(74) + 48).chr }
 
         begin
-            Puppet.config.write(:capass) { |f| f.print pass }
+            Puppet.settings.write(:capass) { |f| f.print pass }
         rescue Errno::EACCES => detail
             raise Puppet::Error, detail.to_s
         end
@@ -217,6 +142,16 @@ class Puppet::SSLCertificates::CA
         }
     end
 
+    # List signed certificates.  This returns a list of hostnames, not actual 
+    # files -- the names can be converted to full paths with host2csrfile. 
+    def list_signed 
+        return Dir.entries(Puppet[:signeddir]).find_all { |file| 
+            file =~ /\.pem$/ 
+        }.collect { |file| 
+            file.sub(/\.pem$/, '') 
+        } 
+    end 
+
     # Create the root certificate.
     def mkrootcert
         # Make the root cert's name the FQDN of the host running the CA.
@@ -238,10 +173,10 @@ class Puppet::SSLCertificates::CA
         Puppet::Util::SUIDManager.asuser(Puppet[:user], Puppet[:group]) do
             @cert = cert.mkselfsigned
         end
-        Puppet.config.write(:cacert) do |f|
+        Puppet.settings.write(:cacert) do |f|
             f.puts @cert.to_pem
         end
-        Puppet.config.write(:capub) do |f|
+        Puppet.settings.write(:capub) do |f|
             f.puts @cert.public_key
         end
         return cert
@@ -259,8 +194,8 @@ class Puppet::SSLCertificates::CA
     # Revoke the certificate with serial number SERIAL issued by this
     # CA. The REASON must be one of the OpenSSL::OCSP::REVOKED_* reasons
     def revoke(serial, reason = OpenSSL::OCSP::REVOKED_STATUS_KEYCOMPROMISE)
-        if @config[:cacrl] == 'none'
-            raise Puppet::Error, "Revocation requires a CRL, but ca_crl is set to 'none'"
+        if @config[:cacrl] == 'false'
+            raise Puppet::Error, "Revocation requires a CRL, but ca_crl is set to 'false'"
         end
         time = Time.now
         revoked = OpenSSL::X509::Revoked.new
@@ -276,7 +211,7 @@ class Puppet::SSLCertificates::CA
     # Take the Puppet config and store it locally.
     def setconfig(hash)
         @config = {}
-        Puppet.config.params("ca").each { |param|
+        Puppet.settings.params("ca").each { |param|
             param = param.intern if param.is_a? String
             if hash.include?(param)
                 @config[param] = hash[param]
@@ -315,7 +250,13 @@ class Puppet::SSLCertificates::CA
             raise Puppet::Error, "CSR sign verification failed"
         end
 
-        serial = File.read(@config[:serial]).chomp.hex
+        serial = nil
+        Puppet.settings.readwritelock(:serial) { |f|
+            serial = File.read(@config[:serial]).chomp.hex
+            # increment the serial
+            f << "%04X" % (serial + 1)
+        }
+
         newcert = Puppet::SSLCertificates.mkcert(
             :type => :server,
             :name => csr.subject,
@@ -325,10 +266,6 @@ class Puppet::SSLCertificates::CA
             :publickey => csr.public_key
         )
 
-        # increment the serial
-        Puppet.config.write(:serial) do |f|
-            f << "%04X" % (serial + 1)
-        end
 
         sign_with_key(newcert)
 
@@ -348,7 +285,7 @@ class Puppet::SSLCertificates::CA
             raise Puppet::Error, "Certificate request for %s already exists" % host
         end
 
-        Puppet.config.writesub(:csrdir, csrfile) do |f|
+        Puppet.settings.writesub(:csrdir, csrfile) do |f|
             f.print csr.to_pem
         end
     end
@@ -364,7 +301,7 @@ class Puppet::SSLCertificates::CA
         end
 
         Puppet::SSLCertificates::Inventory::add(cert)
-        Puppet.config.writesub(:signeddir, certfile) do |f|
+        Puppet.settings.writesub(:signeddir, certfile) do |f|
             f.print cert.to_pem
         end
     end
@@ -407,7 +344,7 @@ class Puppet::SSLCertificates::CA
             @crl = OpenSSL::X509::CRL.new(
                 File.read(@config[:cacrl])
             )
-        elsif @config[:cacrl] == 'none'
+        elsif @config[:cacrl] == 'false'
             @crl = nil
         else
             # Create new CRL
@@ -434,7 +371,7 @@ class Puppet::SSLCertificates::CA
         @crl.next_update = now + 5 * 365*24*60*60
 
         sign_with_key(@crl)
-        Puppet.config.write(:cacrl) do |f|
+        Puppet.settings.write(:cacrl) do |f|
             f.puts @crl.to_pem
         end
     end
@@ -459,4 +396,3 @@ class Puppet::SSLCertificates::CA
     end
 end
 
-# $Id: ca.rb 2259 2007-03-06 19:03:05Z luke $
