@@ -6,30 +6,17 @@ require 'puppet/propertychange'
 
 module Puppet
 class Transaction
-    attr_accessor :component, :resources, :ignoreschedules, :ignoretags
-    attr_accessor :relgraph, :sorted_resources, :configurator
+    attr_accessor :component, :catalog, :ignoreschedules
+    attr_accessor :sorted_resources, :configurator
 
+    # The report, once generated.
     attr_reader :report
+
+    # The list of events generated in this transaction.
+    attr_reader :events
     
-    attr_writer :tags
-
     include Puppet::Util
 
-    Puppet.config.setdefaults(:puppet,
-        :graph => [false, "Whether to create dot graph files for the different
-            configuration graphs.  These dot files can be interpreted by tools
-            like OmniGraffle or dot (which is part of ImageMagick)."],
-        :graphdir => ["$statedir/graphs", "Where to store dot-outputted graphs."]
-    )
-    Puppet.config.setdefaults(:transaction,
-        :tags => ["", "Tags to use to find resources.  If this is set, then
-            only resources tagged with the specified tags will be applied.
-            Values must be comma-separated."],
-        :evaltrace => [false, "Whether each resource should log when it is
-            being evaluated.  This allows you to interactively see exactly
-            what is being done."]
-    )
-
     # Add some additional times for reporting
     def addtimes(hash)
         hash.each do |name, num|
@@ -37,13 +24,14 @@ class Transaction
         end
     end
 
-    # Check to see if we should actually allow deleition.
+    # Check to see if we should actually allow processing, but this really only
+    # matters when a resource is getting deleted.
     def allow_processing?(resource, changes)
         # If a resource is going to be deleted but it still has
         # dependencies, then don't delete it unless it's implicit or the
         # dependency is itself being deleted.
         if resource.purging? and resource.deleting?
-            if deps = @relgraph.dependents(resource) and ! deps.empty? and deps.detect { |d| ! d.deleting? }
+            if deps = relationship_graph.dependents(resource) and ! deps.empty? and deps.detect { |d| ! d.deleting? }
                 resource.warning "%s still depend%s on me -- not purging" %
                     [deps.collect { |r| r.ref }.join(","), deps.length > 1 ? "":"s"] 
                 return false
@@ -53,6 +41,16 @@ class Transaction
         return true
     end
 
+    # Are there any failed resources in this transaction?
+    def any_failed?
+        failures = @failures.inject(0) { |failures, array| failures += array[1]; failures }
+        if failures > 0
+            failures
+        else
+            false
+        end
+    end
+
     # Apply all changes for a resource, returning a list of the events
     # generated.
     def apply(resource)
@@ -83,7 +81,7 @@ class Transaction
         resourceevents = apply_changes(resource, changes)
 
         # If there were changes and the resource isn't in noop mode...
-        unless changes.empty? or changes.include?(:noop)
+        unless changes.empty? or resource.noop
             # Record when we last synced
             resource.cache(:synced, Time.now)
 
@@ -122,7 +120,7 @@ class Transaction
                     puts detail.backtrace
                 end
                 change.property.err "change from %s to %s failed: %s" %
-                    [change.property.is_to_s, change.property.should_to_s, detail]
+                    [change.property.is_to_s(change.is), change.property.should_to_s(change.should), detail]
                 @failures[resource] += 1
                 next
                 # FIXME this should support using onerror to determine
@@ -144,7 +142,10 @@ class Transaction
     # Find all of the changed resources.
     def changed?
         @changes.find_all { |change| change.changed }.collect { |change|
-            change.property.parent
+            unless change.property.resource
+                raise "No resource for %s" % change.inspect
+            end
+            change.property.resource
         }.uniq
     end
     
@@ -152,14 +153,8 @@ class Transaction
     # contained resources might never get cleaned up.
     def cleanup
         if defined? @generated
-            @generated.each do |resource|
-                resource.remove
-            end
+            relationship_graph.remove_resource(*@generated)
         end
-        if defined? @relgraph
-            @relgraph.clear
-        end
-        @resources.clear
     end
 
     # Copy an important relationships from the parent to the newly-generated
@@ -173,10 +168,11 @@ class Transaction
             else
                 edge = [resource, gen_child]
             end
-            unless @relgraph.edge?(edge[1], edge[0])
-                @relgraph.add_edge!(*edge)
+            relationship_graph.add_resource(gen_child) unless relationship_graph.resource(gen_child.ref)
+
+            unless relationship_graph.edge?(edge[1], edge[0])
+                relationship_graph.add_edge(*edge)
             else
-                @relgraph.add_vertex!(gen_child)
                 resource.debug "Skipping automatic relationship to %s" % gen_child
             end
         end
@@ -204,11 +200,7 @@ class Transaction
             end
             
             if children
-                children.each do |child|
-                    child.finish
-                    # Make sure that the vertex is in the relationship graph.
-                    @relgraph.add_vertex!(child)
-                end
+                children.each { |child| child.finish }
                 @generated += children
                 return children
             end
@@ -280,9 +272,12 @@ class Transaction
         # Collect the targets of any subscriptions to those events.  We pass
         # the parent resource in so it will override the source in the events,
         # since eval_generated children can't have direct relationships.
-        @relgraph.matching_edges(events, resource).each do |edge|
-            edge = edge.dup
-            label = edge.label
+        relationship_graph.matching_edges(events, resource).each do |orig_edge|
+            # We have to dup the label here, else we modify the original edge label,
+            # which affects whether a given event will match on the next run, which is,
+            # of course, bad.
+            edge = orig_edge.class.new(orig_edge.source, orig_edge.target)
+            label = orig_edge.label.dup
             label[:event] = events.collect { |e| e.event }
             edge.label = label
             set_trigger(edge)
@@ -297,9 +292,7 @@ class Transaction
     # necessary events.
     def evaluate
         @count = 0
-        
-        graph(@resources, :resources)
-        
+
         # Start logging.
         Puppet::Util::Log.newdestination(@report)
         
@@ -316,7 +309,7 @@ class Transaction
                     ret = eval_resource(resource)
                 end
 
-                if Puppet[:evaltrace]
+                if Puppet[:evaltrace] and @catalog.host_config?
                     resource.info "Evaluated in %0.2f seconds" % seconds
                 end
                 ret
@@ -329,6 +322,7 @@ class Transaction
         Puppet.debug "Finishing transaction %s with %s changes" %
             [self.object_id, @count]
 
+        @events = allevents
         allevents
     end
 
@@ -348,7 +342,7 @@ class Transaction
         # enough to check the immediate dependencies, which is why we use
         # a tree from the reversed graph.
         skip = false
-        deps = @relgraph.dependencies(resource)
+        deps = relationship_graph.dependencies(resource)
         deps.each do |dep|
             if fails = failed?(dep)
                 resource.notice "Dependency %s[%s] has %s failures" %
@@ -362,7 +356,7 @@ class Transaction
     
     # Collect any dynamically generated resources.
     def generate
-        list = @resources.vertices
+        list = @catalog.vertices
         
         # Store a list of all generated resources, so that we can clean them up
         # after the transaction closes.
@@ -384,7 +378,8 @@ class Transaction
                     end
                     made.uniq!
                     made.each do |res|
-                        @resources.add_vertex!(res)
+                        @catalog.add_resource(res)
+                        res.catalog = catalog
                         newlist << res
                         @generated << res
                         res.finish
@@ -409,16 +404,6 @@ class Transaction
             total
         end
 
-        # Unfortunately, RRD does not deal well with changing lists of values,
-        # so we have to pick a list of values and stick with it.  In this case,
-        # that means we record the total time, the config time, and that's about
-        # it.  We should probably send each type's time as a separate metric.
-        @timemetrics.dup.each do |name, value|
-            if Puppet::Type.type(name)
-                @timemetrics.delete(name)
-            end
-        end
-
         # Add all of the metrics related to resource count and status
         @report.newmetric(:resources, @resourcemetrics)
 
@@ -435,30 +420,24 @@ class Transaction
         return @report
     end
 
-    # Produce the graph files if requested.
-    def graph(gr, name)
-        # We don't want to graph the configuration process.
-        return if self.configurator
-        
-        return unless Puppet[:graph]
-
-        file = File.join(Puppet[:graphdir], "%s.dot" % name.to_s)
-        File.open(file, "w") { |f|
-            f.puts gr.to_dot("name" => name.to_s.capitalize)
-        }
+    # Should we ignore tags?
+    def ignore_tags?
+        ! (@catalog.host_config? or Puppet[:name] == "puppet")
     end
 
     # this should only be called by a Puppet::Type::Component resource now
     # and it should only receive an array
     def initialize(resources)
-        if resources.is_a?(Puppet::PGraph)
-            @resources = resources
+        if resources.is_a?(Puppet::Node::Catalog)
+            @catalog = resources
+        elsif resources.is_a?(Puppet::PGraph)
+            raise "Transactions should get catalogs now, not PGraph"
         else
-            @resources = resources.to_graph
+            raise "Transactions require catalogs"
         end
 
         @resourcemetrics = {
-            :total => @resources.vertices.length,
+            :total => @catalog.vertices.length,
             :out_of_sync => 0,    # The number of resources that had changes
             :applied => 0,        # The number of resources fixed
             :skipped => 0,      # The number of resources skipped
@@ -496,70 +475,77 @@ class Transaction
     # Prefetch any providers that support it.  We don't support prefetching
     # types, just providers.
     def prefetch
-        @resources.collect { |obj|
-            if pro = obj.provider
-                pro.class
-            else
-                nil
+        prefetchers = {}
+        @catalog.vertices.each do |resource|
+            if provider = resource.provider and provider.class.respond_to?(:prefetch)
+                prefetchers[provider.class] ||= {}
+                prefetchers[provider.class][resource.title] = resource
             end
-        }.reject { |o| o.nil? }.uniq.each do |klass|
-            # XXX We need to do something special here in case of failure.
-            if klass.respond_to?(:prefetch)
-                klass.prefetch
+        end
+
+        # Now call prefetch, passing in the resources so that the provider instances can be replaced.
+        prefetchers.each do |provider, resources|
+            Puppet.debug "Prefetching %s resources for %s" % [provider.name, provider.resource_type.name]
+            begin
+                provider.prefetch(resources)
+            rescue => detail
+                if Puppet[:trace]
+                    puts detail.backtrace
+                end
+                Puppet.err "Could not prefetch %s provider '%s': %s" % [provider.resource_type.name, provider.name, detail]
             end
         end
     end
     
-    # Prepare to evaluate the elements in a transaction.
+    # Prepare to evaluate the resources in a transaction.
     def prepare
         prefetch()
     
         # Now add any dynamically generated resources
         generate()
-    
-        # Create a relationship graph from our resource graph
-        @relgraph = relationship_graph
         
-        @sorted_resources = @relgraph.topsort
+        # This will throw an error if there are cycles in the graph.
+        @sorted_resources = relationship_graph.topsort
+    end
 
-        # Now make sure no cycles crept into our graph.
-        @relgraph.check_cycle(@sorted_resources)
+    def relationship_graph
+        catalog.relationship_graph
     end
     
-    # Create a graph of all of the relationships in our resource graph.
-    def relationship_graph
-        graph = Puppet::PGraph.new
-        
-        # First create the dependency graph
-        @resources.vertices.each do |vertex|
-            graph.add_vertex!(vertex)
-            vertex.builddepends.each do |edge|
-                graph.add_edge!(edge)
-            end
+    # Send off the transaction report.
+    def send_report
+        begin
+            report = generate_report()
+        rescue => detail
+            Puppet.err "Could not generate report: %s" % detail
+            return
         end
-        
-        # Lastly, add in any autorequires
-        graph.vertices.each do |vertex|
-            vertex.autorequire.each do |edge|
-                unless graph.edge?(edge)
-                    unless graph.edge?(edge.target, edge.source)
-                        vertex.debug "Autorequiring %s" % [edge.source]
-                        graph.add_edge!(edge)
-                    else
-                        vertex.debug "Skipping automatic relationship with %s" % (edge.source == vertex ? edge.target : edge.source)
-                    end
-                end
-            end
-        end
-        
-        graph(graph, :relationships)
-        
-        # Then splice in the container information
-        graph.splice!(@resources, Puppet::Type::Component)
 
-        graph(graph, :expanded_relationships)
+        if Puppet[:rrdgraph] == true
+            report.graph()
+        end
+
+        if Puppet[:summarize]
+            puts report.summary
+        end
         
-        return graph
+        if Puppet[:report]
+            begin
+                reportclient().report(report)
+            rescue => detail
+                Puppet.err "Reporting failed: %s" % detail
+            end
+        end
+    end
+
+    def reportclient
+        unless defined? @reportclient
+            @reportclient = Puppet::Network::Client.report.new(
+                :Server => Puppet[:reportserver]
+            )
+        end
+
+        @reportclient
     end
 
     # Roll all completed changes back.
@@ -589,14 +575,14 @@ class Transaction
             end
             
             # FIXME This won't work right now.
-            @relgraph.matching_edges(events).each do |edge|
+            relationship_graph.matching_edges(events).each do |edge|
                 @targets[edge.target] << edge
             end
 
             # Now check to see if there are any events for this child.
             # Kind of hackish, since going backwards goes a change at a
             # time, not a child at a time.
-            trigger(change.property.parent)
+            trigger(change.property.resource)
 
             # And return the events for collection
             events
@@ -623,7 +609,7 @@ class Transaction
     # Should this resource be skipped?
     def skip?(resource)
         skip = false
-        if ! tagged?(resource)
+        if missing_tags?(resource)
             resource.debug "Not tagged with %s" % tags.join(", ")
         elsif ! scheduled?(resource)
             resource.debug "Not scheduled"
@@ -637,29 +623,27 @@ class Transaction
     
     # The tags we should be checking.
     def tags
-        # Allow the tags to be overridden
         unless defined? @tags
-            @tags = Puppet[:tags]
-        end
-        
-        unless defined? @processed_tags
-            if @tags.nil? or @tags == ""
+            tags = Puppet[:tags]
+            if tags.nil? or tags == ""
                 @tags = []
             else
-                @tags = [@tags] unless @tags.is_a? Array
-                @tags = @tags.collect do |tag|
-                    tag.split(/\s*,\s*/)
-                end.flatten
+                @tags = tags.split(/\s*,\s*/)
             end
-            @processed_tags = true
         end
         
         @tags
     end
+
+    def tags=(tags)
+        tags = [tags] unless tags.is_a?(Array)
+        @tags = tags
+    end
     
     # Is this resource tagged appropriately?
-    def tagged?(resource)
-        self.ignoretags or tags.empty? or resource.tagged?(tags)
+    def missing_tags?(resource)
+        return false if self.ignore_tags? or tags.empty?
+        return true unless resource.tagged?(tags)
     end
     
     # Are there any edges that target this resource?
@@ -754,4 +738,3 @@ end
 
 require 'puppet/transaction/report'
 
-# $Id: transaction.rb 2372 2007-03-30 00:47:03Z luke $
